import 'dart:async';

import 'package:equatable/equatable.dart';
import 'package:firebase_auth/firebase_auth.dart';
import 'package:trhung_account_config/account.dart';
import 'package:trhung_account_config/account_repository.dart';
import 'package:flutter_bloc/flutter_bloc.dart';

part 'auth_event.dart';
part 'auth_state.dart';

class AuthBloc extends Bloc<AuthEvent, AuthState> {
  AuthBloc(this.accountRepository) : super(AuthInitial());

  final AccountRepository accountRepository;

  String errorMessage = 'Please check Email or Password again!!!';

  @override
  Stream<AuthState> mapEventToState(
    AuthEvent event,
  ) async* {
    // yield AuthLoading();
    if (event is AuthenticationStarted) {
      final isSignedIn = await accountRepository.isSignedIn();
      if (isSignedIn) {
        try {
          final account = await accountRepository.getUserInformation();
          if (account.isExpire) {
            yield AuthFail(
                'Your account was over date. Please make sure you have upgrade your license. If you have any concern, please contact to either your IT departement or LEAN VIETNAM through hotline 093 8763 289 for more information.');
          } else {
            if (account.user.emailVerified) {
              yield AuthInfo(account);
            } else {
              yield AuthFail('Check your mailbox for verification');
            }
          }
        } on AccountError {
          yield AuthFail(errorMessage);
        }
      } else {
        yield NotLoginYet();
      }
    } else if (event is SignIn) {
      if (isAccountValid(event.email, event.pass)) {
        try {
          final authResult =
              await accountRepository.signIn(event.email, event.pass);
          if (authResult == null) {
            yield AuthFail(
                'Wrong password or this Account have not Sign-Up yet');
          } else {
            if (!authResult.user.emailVerified) {
              yield AuthFail('Check your mailbox for verification');
            } else {
              final account = await accountRepository.getUserInformation();
              if (account.isExpire) {
                yield AuthFail(
                    'Your account was over date or deactive. Please make sure you have upgrade your license. If you have any concern, please contact to either your IT departement or LEAN VIETNAM through hotline 093 8763 289 for more information.');
              } else {
                yield AuthInfo(account);
              }
            }
          }
        } on AccountError {
          yield AuthFail('AccountError');
        }
      } else {
        yield AuthFail('Email and password can not be empty');
      }
    } else if (event is SignUp) {
      try {
        final authResult =
            await accountRepository.signUp(event.email, event.pass);
        if (authResult == null) {
          yield AuthFail(errorMessage);
        } else {
          yield AuthInitial();
        }
      } on AccountError {
        yield AuthFail(errorMessage);
      }
    } else if (event is SignOut) {
      try {
        await accountRepository.signOut();
        yield AuthSignOut('Log out!');
      } on AccountError {
        yield AuthFail(errorMessage);
      }
    } else if (event is GetAuthInfo) {
      try {
        final account = await accountRepository.getUserInformation();
        if (account.isExpire) {
          yield AuthFail(
              'Your account was over date. Please make sure you have upgrade your license. If you have any concern, please contact to either your IT departement or LEAN VIETNAM through hotline 093 8763 289 for more information.');
        } else {
          yield AuthInfo(account);
        }
      } on AccountError {
        yield AuthFail(errorMessage);
      }
    } else if (event is PasswordReset) {
      try {
        if (event.email == null || event.email.length == 0) {
          yield AuthFail('Please enter Email address for password reset');
        } else {
          final mess = await accountRepository.resetPassword(event.email);
          yield AuthResetPassword(mess);
        }
      } on AccountError {
        yield AuthFail('Error!!!');
      }
    }
  }

  bool isAccountValid(String email, String pass) {
    if (email == null ||
        !email.contains('@') ||
        email.length == 0 ||
        pass == null ||
        pass.length == 0) {
      return false;
    } else {
      return true;
    }
  }
}
