import 'package:cloud_firestore/cloud_firestore.dart';
import 'package:firebase_auth/firebase_auth.dart';
import 'package:trhung_account_config/account.dart';
import 'package:intl/intl.dart';

class FirAuth {
  final FirebaseAuth _fireBaseAuth = FirebaseAuth.instance;
  UserCredential authResult;

  DocumentSnapshot usersData;

  Account account = new Account();

  Future<bool> isSignedIn() async {
    final currentUser = _fireBaseAuth.currentUser;
    Stream<User> snapshot = _fireBaseAuth.authStateChanges();
    snapshot.listen((currentUser) {
      return currentUser != null;
    });
    return currentUser != null;
  }

  Future<UserCredential> signIn(String email, String pass) async {
    await _fireBaseAuth
        .signInWithEmailAndPassword(email: email.trim(), password: pass)
        .then((user) {
      authResult = user;
      return authResult;
    }).catchError((err) {
      print("err: " + err.toString());
      authResult = null;
    });
    return authResult;
  }

  Future<UserCredential> signUp(String email, String pass) async {
    await _fireBaseAuth
        .createUserWithEmailAndPassword(email: email, password: pass)
        .then((user) {
      authResult = user;
      _createUser(user.user.uid, email, pass);
      try {
        user.user.sendEmailVerification();
      } catch (e) {
        print("An error occured while trying to send email verification");
        print(e.message);
      }
    }).catchError((err) {
      print("err: " + err.toString());
      authResult = null;
    });
    return authResult;
  }

  _createUser(String userId, String email, String pass) {
    FirebaseFirestore.instance.collection('users').doc(userId).set({
      'email': email,
      'name': 'account.name',
      'accountLevel': 'account.accountLevel',
      'level': 'account.level',
      'subDepartment': 'account.subDepartment',
      'department': 'account.department',
      'company': 'account.company',
      'signUpDate': DateFormat('yyyy-MM-dd').format(DateTime.now()),
      'expiredDate': DateFormat('yyyy-MM-dd')
          .format(DateTime.now().add(new Duration(days: 0))),
      'purchaseLevel': '0'
    }).catchError((onError) {
      print(onError.toString());
    }).whenComplete(() {
      print('OK');
    });
  }

  Future<String> resetPassword(String email) async {
    await _fireBaseAuth.sendPasswordResetEmail(email: email).whenComplete(() {
      return 'Email has been sent';
    }).catchError((error) {
      return 'Email has been sent wrong';
    });
    return 'Email has been sent, please check at your mailbox';
  }

  Future<void> signOut() async {
    print("signOut");
    return _fireBaseAuth.signOut();
  }

  Future<Account> getUserInformation() async {
    final currentUser = _fireBaseAuth.currentUser;
    if (currentUser == null) {
      return null;
    } else {
      await FirebaseFirestore.instance
          .collection("users")
          .doc(currentUser.uid)
          .get()
          .then((DocumentSnapshot userData) {
        if (userData != null) {
          account.email = currentUser.email;
          account.name = userData.data()['name'];
          account.accountLevel = userData.data()['accountLevel'];
          account.level = userData.data()['level'];
          account.department = userData.data()['department'];
          account.subDepartment = userData.data()['subDepartment'];
          account.company = userData.data()['company'];
          account.signUpDate = userData.data()['signUpDate'];
          account.expiredDate = userData.data()['expiredDate'];
          account.purchaseLevel = userData.data()['purchaseLevel'];
        }
        // print(account.email);
      });
      account.user = currentUser;
    }

    return account;
  }
}
